@@ -21,27 +21,6 @@ from metalsmith import _instance
 from metalsmith import _os_api
 
 
-class TestInit(testtools.TestCase):
-    def test_missing_auth(self):
-        self.assertRaisesRegex(TypeError, 'must be provided', _os_api.API)
-
-    def test_both_provided(self):
-        self.assertRaisesRegex(TypeError, 'not both', _os_api.API,
-                               session=mock.Mock(), cloud_region=mock.Mock())
-
-    def test_session_only(self):
-        session = mock.Mock()
-        api = _os_api.API(session=session)
-        self.assertIs(api.session, session)
-
-    @mock.patch.object(_os_api.connection, 'Connection', autospec=True)
-    def test_cloud_region_only(self, mock_conn):
-        region = mock.Mock()
-        api = _os_api.API(cloud_region=region)
-        self.assertIs(api.session, region.get_session.return_value)
-        mock_conn.assert_called_once_with(config=region)
-
-
 class TestNodes(testtools.TestCase):
     def setUp(self):
         super(TestNodes, self).setUp()
@@ -50,12 +29,11 @@ class TestNodes(testtools.TestCase):
             fixtures.MockPatchObject(_os_api.ir_client, 'get_client',
                                      autospec=True))
         self.cli = self.ironic_fixture.mock.return_value
-        self.api = _os_api.API(session=self.session)
+        self.api = _os_api.API(session=self.session, connection=mock.Mock())
 
     def test_get_node_by_uuid(self):
         res = self.api.get_node('uuid1')
-        self.cli.node.get.assert_called_once_with('uuid1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('uuid1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_get_node_by_hostname(self):
@@ -66,8 +44,7 @@ class TestNodes(testtools.TestCase):
         ]
         res = self.api.get_node('host1', accept_hostname=True)
         # Loading details
-        self.cli.node.get.assert_called_once_with('uuid1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('uuid1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_get_node_by_hostname_not_found(self):
@@ -78,8 +55,7 @@ class TestNodes(testtools.TestCase):
         ]
         res = self.api.get_node('host1', accept_hostname=True)
         # Loading details
-        self.cli.node.get.assert_called_once_with('host1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('host1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_get_node_by_node(self):
@@ -90,8 +66,7 @@ class TestNodes(testtools.TestCase):
     def test_get_node_by_node_with_refresh(self):
         res = self.api.get_node(mock.Mock(spec=['uuid'], uuid='uuid1'),
                                 refresh=True)
-        self.cli.node.get.assert_called_once_with('uuid1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('uuid1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_get_node_by_instance(self):
@@ -104,8 +79,7 @@ class TestNodes(testtools.TestCase):
         inst = _instance.Instance(mock.Mock(),
                                   mock.Mock(spec=['uuid'], uuid='uuid1'))
         res = self.api.get_node(inst, refresh=True)
-        self.cli.node.get.assert_called_once_with('uuid1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('uuid1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_find_node_by_hostname(self):
@@ -116,8 +90,7 @@ class TestNodes(testtools.TestCase):
         ]
         res = self.api.find_node_by_hostname('host1')
         # Loading details
-        self.cli.node.get.assert_called_once_with('uuid1',
-                                                  fields=_os_api.NODE_FIELDS)
+        self.cli.node.get.assert_called_once_with('uuid1')
         self.assertIs(res, self.cli.node.get.return_value)
 
     def test_find_node_by_hostname_cached(self):
