@@ -113,35 +113,35 @@ class TestScheduleNode(testtools.TestCase):
 class TestCapabilitiesFilter(testtools.TestCase):
 
     def test_fail_no_capabilities(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {'profile': 'compute'})
+        fltr = _scheduler.CapabilitiesFilter({'profile': 'compute'})
         self.assertRaisesRegex(exceptions.CapabilitiesNotFound,
                                'No available nodes found with capabilities '
                                'profile=compute, existing capabilities: none',
                                fltr.fail)
 
     def test_nothing_requested_nothing_found(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {})
+        fltr = _scheduler.CapabilitiesFilter({})
         node = mock.Mock(properties={}, spec=['properties', 'name', 'uuid'])
         self.assertTrue(fltr(node))
 
     def test_matching_node(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {'profile': 'compute',
-                                                     'foo': 'bar'})
+        fltr = _scheduler.CapabilitiesFilter({'profile': 'compute',
+                                              'foo': 'bar'})
         node = mock.Mock(
             properties={'capabilities': 'foo:bar,profile:compute,answer:42'},
             spec=['properties', 'name', 'uuid'])
         self.assertTrue(fltr(node))
 
     def test_not_matching_node(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {'profile': 'compute',
-                                                     'foo': 'bar'})
+        fltr = _scheduler.CapabilitiesFilter({'profile': 'compute',
+                                              'foo': 'bar'})
         node = mock.Mock(
             properties={'capabilities': 'foo:bar,answer:42'},
             spec=['properties', 'name', 'uuid'])
         self.assertFalse(fltr(node))
 
     def test_fail_message(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {'profile': 'compute'})
+        fltr = _scheduler.CapabilitiesFilter({'profile': 'compute'})
         node = mock.Mock(
             properties={'capabilities': 'profile:control'},
             spec=['properties', 'name', 'uuid'])
@@ -153,7 +153,7 @@ class TestCapabilitiesFilter(testtools.TestCase):
                                fltr.fail)
 
     def test_malformed_capabilities(self):
-        fltr = _scheduler.CapabilitiesFilter('rsc', {'profile': 'compute'})
+        fltr = _scheduler.CapabilitiesFilter({'profile': 'compute'})
         for cap in ['foo,profile:control', 42, 'a:b:c']:
             node = mock.Mock(properties={'capabilities': cap},
                              spec=['properties', 'name', 'uuid'])
@@ -164,78 +164,71 @@ class TestCapabilitiesFilter(testtools.TestCase):
                                fltr.fail)
 
 
-class TestValidationFilter(testtools.TestCase):
+class TestTraitsFilter(testtools.TestCase):
 
-    def setUp(self):
-        super(TestValidationFilter, self).setUp()
-        self.api = mock.Mock(spec=['validate_node'])
-        self.fltr = _scheduler.ValidationFilter(self.api, 'rsc',
-                                                {'profile': 'compute'})
+    def test_fail_no_traits(self):
+        fltr = _scheduler.TraitsFilter(['tr1', 'tr2'])
+        self.assertRaisesRegex(exceptions.TraitsNotFound,
+                               'No available nodes found with traits '
+                               'tr1, tr2, existing traits: none',
+                               fltr.fail)
 
-    def test_pass(self):
-        node = mock.Mock(spec=['uuid', 'name'])
-        self.assertTrue(self.fltr(node))
+    def test_no_traits(self):
+        fltr = _scheduler.TraitsFilter([])
+        node = mock.Mock(spec=['name', 'uuid'])
+        self.assertTrue(fltr(node))
 
-    def test_fail_validation(self):
-        node = mock.Mock(spec=['uuid', 'name'])
-        self.api.validate_node.side_effect = RuntimeError('boom')
-        self.assertFalse(self.fltr(node))
+    def test_ok(self):
+        fltr = _scheduler.TraitsFilter(['tr1', 'tr2'])
+        node = mock.Mock(spec=['name', 'uuid', 'traits'],
+                         traits=['tr3', 'tr2', 'tr1'])
+        self.assertTrue(fltr(node))
 
-        self.assertRaisesRegex(exceptions.ValidationFailed,
-                               'All available nodes have failed validation: '
-                               'Node .* failed validation: boom',
-                               self.fltr.fail)
+    def test_missing_one(self):
+        fltr = _scheduler.TraitsFilter(['tr1', 'tr2'])
+        node = mock.Mock(spec=['name', 'uuid', 'traits'],
+                         traits=['tr3', 'tr1'])
+        self.assertFalse(fltr(node))
+
+    def test_missing_all(self):
+        fltr = _scheduler.TraitsFilter(['tr1', 'tr2'])
+        node = mock.Mock(spec=['name', 'uuid', 'traits'], traits=None)
+        self.assertFalse(fltr(node))
 
 
-@mock.patch.object(_scheduler, 'ValidationFilter', autospec=True)
 class TestIronicReserver(testtools.TestCase):
 
     def setUp(self):
         super(TestIronicReserver, self).setUp()
         self.node = mock.Mock(spec=['uuid', 'name'])
-        self.api = mock.Mock(spec=['reserve_node', 'release_node'])
+        self.api = mock.Mock(spec=['reserve_node', 'release_node',
+                                   'validate_node'])
         self.api.reserve_node.side_effect = lambda node, instance_uuid: node
-        self.reserver = _scheduler.IronicReserver(self.api, 'rsc', {})
+        self.reserver = _scheduler.IronicReserver(self.api)
 
-    def test_fail(self, mock_validation):
-        self.assertRaisesRegex(exceptions.AllNodesReserved,
+    def test_fail(self):
+        self.assertRaisesRegex(exceptions.NoNodesReserved,
                                'All the candidate nodes are already reserved',
                                self.reserver.fail)
 
-    def test_ok(self, mock_validation):
+    def test_ok(self):
         self.assertEqual(self.node, self.reserver(self.node))
+        self.api.validate_node.assert_called_with(self.node)
         self.api.reserve_node.assert_called_once_with(
             self.node, instance_uuid=self.node.uuid)
-        mock_validation.return_value.assert_called_once_with(self.node)
 
-    def test_reservation_failed(self, mock_validation):
+    def test_reservation_failed(self):
         self.api.reserve_node.side_effect = RuntimeError('conflict')
         self.assertRaisesRegex(RuntimeError, 'conflict',
                                self.reserver, self.node)
+        self.api.validate_node.assert_called_with(self.node)
         self.api.reserve_node.assert_called_once_with(
             self.node, instance_uuid=self.node.uuid)
-        self.assertFalse(mock_validation.return_value.called)
 
-    def test_validation_failed(self, mock_validation):
-        mock_validation.return_value.return_value = False
-        mock_validation.return_value.fail.side_effect = RuntimeError('fail')
-        self.assertRaisesRegex(RuntimeError, 'fail',
+    def test_validation_failed(self):
+        self.api.validate_node.side_effect = RuntimeError('fail')
+        self.assertRaisesRegex(exceptions.ValidationFailed, 'fail',
                                self.reserver, self.node)
-        self.api.reserve_node.assert_called_once_with(
-            self.node, instance_uuid=self.node.uuid)
-        mock_validation.return_value.assert_called_once_with(self.node)
-        self.api.release_node.assert_called_once_with(self.node)
-
-    @mock.patch.object(_scheduler.LOG, 'exception', autospec=True)
-    def test_validation_and_release_failed(self, mock_log_exc,
-                                           mock_validation):
-        mock_validation.return_value.return_value = False
-        mock_validation.return_value.fail.side_effect = RuntimeError('fail')
-        self.api.release_node.side_effect = Exception()
-        self.assertRaisesRegex(RuntimeError, 'fail',
-                               self.reserver, self.node)
-        self.api.reserve_node.assert_called_once_with(
-            self.node, instance_uuid=self.node.uuid)
-        mock_validation.return_value.assert_called_once_with(self.node)
-        self.api.release_node.assert_called_once_with(self.node)
-        self.assertTrue(mock_log_exc.called)
+        self.api.validate_node.assert_called_once_with(self.node)
+        self.assertFalse(self.api.reserve_node.called)
+        self.assertFalse(self.api.release_node.called)
